/*
 *      File:           uc.h
 *      Contents:       Constants, defines, global variables, and function
 *			prototypes.
 */


/* INCLUDE FILES */
#include <stdio.h>
#include <math.h>
#include "args.h"


/* Directions on a compass that we can move around in the image */
#define MAXDIRECT 	8
#define	NW		0
#define N		1                  /* ------------------- */
#define	NE		2                  /* |  0  |  1  |  2  | */
#define	W		3 		   /* |-----+-----+-----| */
#define	SW		4		   /* |  3  |  x  |  7  | */
#define S		5		   /* |-----+-----+-----| */
#define	SE		6		   /* |  4  |  5  |  6  | */
#define	E		7                  /* ------------------- */
#define PI		M_PI


/* Constants used to mark the image during edge detection */
#define MAXIMA		0x80
#define VISITED		0x40
#define JUNCTION	0x20
#define NOTUSED		0x10


/* Edge types: */
#define	NONCIRCULAR	0	/* Start and end are not coincident */
#define	CIRCULAR	1	/* Start and end are coincident     */


/* Number of entries in the edgel hash-table */
#define SIZE_HASH_TABLE   20000


/* Width of mask in terms of simgas */
#define MASK_SIGMAS       4.0


/* Cartesian position */
typedef struct position_s
  {
  double	x, y ;
  } position ;


/* Gradient */
typedef struct gradient_s
  {
  double		p, q ;
  } gradient ;


/* Edgels */
typedef struct edgel_s
  {
  position		pos ;	/* X,Y Position in Cartiesian space */
  gradient		grad ;	/* X,Y Gradient values */
  position		loc ;	/* Localized X,Y in Cartiesian space */
  int			high ;	/* TRUE if above high threshold. */
  struct edgel_s	*prev ;	/* Pointer to previous edgel */ 
  struct edgel_s	*next ;	/* Pointer to next edgel */
  } edgel;


/* Junctions */
typedef struct junction_s
  {
  position	pos ;    /* X,Y Position in Cartiesian space */
  gradient	grad ;   /* X,Y Gradient values */
  position	loc ;	 /* Localized X,Y in Cartiesian space */
  edgel 	*neighbors[MAXDIRECT] ; /* Pointers to possible neighbors */
  } junction ;


/* List of Junctions */
typedef struct junction_list_s
  {
  junction			*jct ;	/* A pointer to a junction */
  struct junction_list_s	*next ;	/* Pointer to the next junction */
  } junction_list ;


/* Edges */
typedef struct edge_s 
  {
  int		type ;		/* tells us if the edge is circular or not */
  edgel		*start_e ;	/* Starting pointer of an edgel */
  edgel		*end_e ;	/* Ending pointer of an edgel */
  junction	*start_j ;	/* Starting pointer of a junction */
  junction	*end_j ;	/* Ending pointer of a junction */
  double	length ;	/* Length of an edge */
  struct edge_s *prev ;		/* Pointer to previous connected edge */
  struct edge_s *next ;		/* Pointer to next connected edge */
  } edge ;

junction_list *jct_lst ;


/* CONSTANTS */
#define FAILURE 1
#define SUCCESS 0

#define	MIN_FLOAT	-1e126
#define	MAX_FLOAT	1e127

#define	X	0
#define	Y	1
#define	Z	2
#define	DIMS	3

/* Maxima */
#define HEADERSIZE	0
#define	MAXLINELEN	1024
#define	MAXPATHLEN	256
#define MAXNAMELEN	80

#define	SPEC_BASE	0


/* MACROS */

#define MIN(a,b)	(((a)<(b))?(a):(b))
#define ABS(a)		(((a)>=0)?(a):(-(a)))
#define MAX(a,b)	(((a)>(b))?(a):(b))
#define SQR(a)		((a)*(a))
#define	VALINC(v)	(((v) == NULL) ? 0.0 : *((v)++))
#define	VALOFF(v,o)	(((v) == NULL) ? 0.0 : *(v+o))


/* Global variables */
/* extern char 		outfile[MAXPATHLEN] ;

extern double		sigma;
extern int		mask;
extern double		low_threshold;
extern double		high_threshold;
extern int		XSIZE;
extern int		YSIZE;

extern int		Sub_pixel;
extern int		NUM_EDGELS;
extern int		NUM_EDGES;
extern int		NUM_JCTS;

extern unsigned short	*Record_image ;
extern unsigned short	*image;
extern unsigned short	*timage;

extern edgel		**edgels ;
extern edgel		*tedgels ;

extern edge		*edges ;
extern edge		*tedges ; */
